@@ -115,6 +115,12 @@ impl ByValueChecker {
                         .results
                         .insert(api.name().clone(), StructDetails::new(PodState::IsPod));
                 }
+                Api::ExternCppType { .. } => {
+                    byvalue_checker.results.insert(
+                        api.name().clone(),
+                        StructDetails::new(PodState::SafeToBePod),
+                    );
+                }
                 _ => {}
             }
         }
