@@ -495,6 +495,14 @@ fn create_type_database() -> TypeDatabase {
         true,
         true,
     ));
+    db.insert(TypeDetails::new(
+        "usize",
+        "size_t",
+        Behavior::CByValue,
+        None,
+        true,
+        true,
+    ));
     db.insert(TypeDetails::new(
         "autocxx::c_void",
         "void",
