@@ -7,82 +7,78 @@
 // except according to those terms.
 
 use crate::ast_discoverer::{Discoveries, DiscoveryErr};
-use crate::CppCodegenOptions;
 use crate::{
     cxxbridge::CxxBridge, Error as EngineError, GeneratedCpp, IncludeCppEngine,
     RebuildDependencyRecorder,
 };
+use crate::{CppCodegenOptions, LocatedSynError};
 use autocxx_parser::directives::SUBCLASS;
 use autocxx_parser::{AllowlistEntry, RustPath, Subclass, SubclassAttrs};
-use proc_macro2::{Span, TokenStream};
+use miette::Diagnostic;
+use proc_macro2::TokenStream;
 use quote::ToTokens;
-use std::{collections::HashSet, fmt::Display, io::Read, path::PathBuf};
+use std::{collections::HashSet, io::Read, path::PathBuf};
 use std::{panic::UnwindSafe, path::Path, rc::Rc};
 use syn::{token::Brace, Item, ItemMod};
+use thiserror::Error;
 
 /// Errors which may occur when parsing a Rust source file to discover
 /// and interpret include_cxx macros.
-#[derive(Debug)]
+#[derive(Error, Diagnostic, Debug)]
 pub enum ParseError {
-    /// Unable to open the source file
+    #[error("unable to open the source file: {0}")]
     FileOpen(std::io::Error),
-    /// The .rs file couldn't be read.
+    #[error("the .rs file couldn't be read: {0}")]
     FileRead(std::io::Error),
-    /// The .rs file couldn't be parsed.
-    Syntax(syn::Error),
+    #[error("syntax error interpreting Rust code: {0}")]
+    #[diagnostic(transparent)]
+    Syntax(LocatedSynError),
+    #[error("generate!/generate_ns! was used at the same time as generate_all!")]
+    ConflictingAllowlist,
+    #[error("the subclass attribute couldn't be parsed: {0}")]
+    #[diagnostic(transparent)]
+    SubclassSyntax(LocatedSynError),
     /// The include CPP macro could not be expanded into
     /// Rust bindings to C++, because of some problem during the conversion
     /// process. This could be anything from a C++ parsing error to some
     /// C++ feature that autocxx can't yet handle and isn't able to skip
     /// over. It could also cover errors in your syntax of the `include_cpp`
     /// macro or the directives inside.
+    #[error("the include_cpp! macro couldn't be expanded into Rust bindings to C++: {0}")]
+    #[diagnostic(transparent)]
     AutocxxCodegenError(EngineError),
     /// There are two or more `include_cpp` macros with the same
     /// mod name.
+    #[error("there are two or more include_cpp! mods with the same mod name")]
     ConflictingModNames,
+    #[error("dynamic discovery was enabled but no mod was found")]
     ZeroModsForDynamicDiscovery,
+    #[error("dynamic discovery was enabled but multiple mods were found")]
     MultipleModsForDynamicDiscovery,
+    #[error("a problem occurred while discovering C++ APIs used within the Rust: {0}")]
     Discovery(DiscoveryErr),
 }
 
-impl Display for ParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self {
-            ParseError::FileOpen(err) => write!(f, "Unable to open file: {}", err)?,
-            ParseError::FileRead(err) => write!(f, "Unable to read file: {}", err)?,
-            ParseError::Syntax(err) => write!(f, "Syntax error parsing Rust file: {}", err)?,
-            ParseError::AutocxxCodegenError(err) =>
-                write!(f, "Unable to parse include_cpp! macro: {}", err)?,
-            ParseError::ConflictingModNames =>
-                write!(f, "There are two or more include_cpp! macros with the same output mod name. Use name!")?,
-            ParseError::ZeroModsForDynamicDiscovery =>
-                write!(f, "This file contains extra information to append to an include_cpp! but no such include_cpp! was found in this file.")?,
-            ParseError::MultipleModsForDynamicDiscovery =>
-                write!(f, "This file contains extra information to append to an include_cpp! but multiple such include_cpp! declarations were found in this file.")?,
-            ParseError::Discovery(DiscoveryErr::FoundExternRustFunOnTypeWithoutClearReceiver) => write!(f, "#[extern_rust_function] was attached to a method in an impl block that was too complex for autocxx. autocxx supports only \"impl X {{...}}\" where X is a single identifier, not a path or more complex type.")?,
-            ParseError::Discovery(DiscoveryErr::NoParameterOnMethod) => write!(f, "#[extern_rust_function] was attached to a method taking no parameters.")?,
-            ParseError::Discovery(DiscoveryErr::NonReferenceReceiver) => write!(f, "#[extern_rust_function] was attached to a method taking a receiver by value.")?,
-            ParseError::Discovery(DiscoveryErr::FoundExternRustFunWithinMod) => write!(f, "#[extern_rust_function] was in an impl block nested wihtin another block. This is only supported in the outermost mod of a file, alongside the include_cpp!.")?,
-        }
-        Ok(())
-    }
-}
-
 /// Parse a Rust file, and spot any include_cpp macros within it.
 pub fn parse_file<P1: AsRef<Path>>(
     rs_file: P1,
     auto_allowlist: bool,
 ) -> Result<ParsedFile, ParseError> {
-    let mut source = String::new();
+    let mut source_code = String::new();
     let mut file = std::fs::File::open(rs_file).map_err(ParseError::FileOpen)?;
-    file.read_to_string(&mut source)
+    file.read_to_string(&mut source_code)
         .map_err(ParseError::FileRead)?;
     proc_macro2::fallback::force();
-    let source = syn::parse_file(&source).map_err(ParseError::Syntax)?;
-    parse_file_contents(source, auto_allowlist)
+    let source = syn::parse_file(&source_code)
+        .map_err(|e| ParseError::Syntax(LocatedSynError::new(e, &source_code)))?;
+    parse_file_contents(source, auto_allowlist, &source_code)
 }
 
-fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<ParsedFile, ParseError> {
+fn parse_file_contents(
+    source: syn::File,
+    auto_allowlist: bool,
+    file_contents: &str,
+) -> Result<ParsedFile, ParseError> {
     #[derive(Default)]
     struct State {
         auto_allowlist: bool,
@@ -91,7 +87,12 @@ fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<Parsed
         discoveries: Discoveries,
     }
     impl State {
-        fn parse_item(&mut self, item: Item, mod_path: Option<RustPath>) -> Result<(), ParseError> {
+        fn parse_item(
+            &mut self,
+            item: Item,
+            mod_path: Option<RustPath>,
+            file_contents: &str,
+        ) -> Result<(), ParseError> {
             let result = match item {
                 Item::Macro(mac)
                     if mac
@@ -103,7 +104,7 @@ fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<Parsed
                         .unwrap_or(false) =>
                 {
                     Segment::Autocxx(
-                        crate::IncludeCppEngine::new_from_syn(mac.mac)
+                        crate::IncludeCppEngine::new_from_syn(mac.mac, file_contents)
                             .map_err(ParseError::AutocxxCodegenError)?,
                     )
                 }
@@ -126,7 +127,7 @@ fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<Parsed
                             Some(mod_path) => mod_path.append(itm.ident.clone()),
                         };
                         for item in items {
-                            mod_state.parse_item(item, Some(mod_path.clone()))?
+                            mod_state.parse_item(item, Some(mod_path.clone()), file_contents)?
                         }
                         self.extra_superclasses.extend(mod_state.extra_superclasses);
                         self.discoveries.extend(mod_state.discoveries);
@@ -156,9 +157,13 @@ fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<Parsed
                     if let Some(is_superclass_attr) = is_superclass_attr {
                         if !is_superclass_attr.tokens.is_empty() {
                             let subclass = its.ident.clone();
-                            let args: SubclassAttrs = is_superclass_attr
-                                .parse_args()
-                                .map_err(ParseError::Syntax)?;
+                            let args: SubclassAttrs =
+                                is_superclass_attr.parse_args().map_err(|e| {
+                                    ParseError::SubclassSyntax(LocatedSynError::new(
+                                        e,
+                                        file_contents,
+                                    ))
+                                })?;
                             if let Some(superclass) = args.superclass {
                                 self.extra_superclasses.push(Subclass {
                                     superclass,
@@ -188,7 +193,7 @@ fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<Parsed
         ..Default::default()
     };
     for item in source.items {
-        state.parse_item(item, None)?
+        state.parse_item(item, None, file_contents)?
     }
     let State {
         auto_allowlist,
@@ -221,8 +226,8 @@ fn parse_file_contents(source: syn::File, auto_allowlist: bool) -> Result<Parsed
                         engine
                             .config_mut()
                             .allowlist
-                            .push(AllowlistEntry::Item(cpp), Span::call_site())
-                            .map_err(ParseError::Syntax)?;
+                            .push(AllowlistEntry::Item(cpp))
+                            .map_err(|_| ParseError::ConflictingAllowlist)?;
                     }
                 }
                 engine
